package RSSReader::Gtk2UI::Category;
use strict;
use RSSReader::Category;
use RSSReader::Gtk2UI::Category::InfoPane;
use RSSReader::Gtk2UI::Category::MessageList;
use RSSReader::Gtk2UI::Message;
use Data::Dumper;
use base qw(RSSReader::Logger);

our $_cats;

sub new {
	my $c = shift;
	my %a = @_;

	if (exists $_cats->{$a{id}}) { return $_cats->{$a{id}}; }
		
	#otherwise .. fetch the cat and build the widget... after this
	# only redraws are needed.

	my $cat = RSSReader::Category->new(id=>$a{id});
	my $s = bless { _cat => $cat, id => $a{id} }, $c;
	$s->{_prefs} = RSSReader::Gtk2UI::Preferences->new();

	$_cats->{$a{id}} = $s;
	#TODO Register em updates da categoria, redraw proposals

	$s->{_box} 		= new Gtk2::VBox;
	$s->{_widget}		= new Gtk2::VPaned;
	$s->{_widget}->add1($s->{_box});

	$s->_redraw();
	$s->show();

	return $s;
}

sub get_category { return shift->{_cat} }

sub show {
       	my $s = shift;
       	$s->{_message_list}->cleanup if defined($s->{_message_list});
    	$s->{_message_list}->show;
       	my $position;
       	if ($s->{_prefs}->per_view_pane()) {
       		$position = $s->{_prefs}->get('category_view_'.$s->{_cat}->id(), 'pane_heigth') || 350;
       	}
       	else {
        	$position = $s->{_prefs}->get('all_views_global', 'pane_heigth') || 350;
       	}
	$s->{_widget}->set_position($position);
        $s->{_widget}->show_all();
}

sub widget {
	my $s = shift;
	return $s->{_widget};
}

sub id {
	my $s = shift;
	return $s->{id};
}


sub _redraw {
	my $s = shift;

	if (! $s->{_redrawed} ) { 
        	$s->{_panel} = RSSReader::Gtk2UI::Category::InfoPane->new( id=>$s->{_cat}->id );
		$s->{_box}->pack_start($s->{_panel}->widget(),0,0,0);

		#get handle_position
		$s->{_widget}->show_all();

		my $mlist = RSSReader::Gtk2UI::Category::MessageList->new(id => $s->{_cat}->id );
		$s->{_message_list} = $mlist;
		$s->{_message_list}->widget()->show();
		$s->{_message_list}->notify_me(event=>'message_selected', caller=>$s, call=>'message_selected');

		$s->{_box}->pack_start($mlist->widget(),1,1,0);

		$s->{_current_message} = RSSReader::Gtk2UI::Message->new;
		$s->{_widget}->add2($s->{_current_message}->widget);
		
		$s->{_redrawed} = 1;
	}

	$s->{_widget}->show();
}

sub message_selected {
        my $s = shift;
	my %a = @_;

	$s->{_current_message}->load_message(message=>$a{message});
	$a{message}->mark_read;
	$s->{_message_selected} = $a{message};
}

sub has_unread_messages  { 
	my $s = shift; 
	my %a = @_;
	if ($a{scroll} && $s->{_message_selected}) {
                return 1 unless ( $s->{_current_message}->full_scrolled() );
        }
	return scalar( @{$s->{_cat}->unread_messages()} ) 
}

sub select_next_unread   { 
	my $s = shift; 
	my %a = @_;
	if ($a{scroll} && $s->{_message_selected}) {
		return $s->{_current_message}->scroll 
			unless ( $s->{_current_message}->full_scrolled() );
        }
	return $s->{_message_list}->select_next_unread 
}

sub can_fetch            { return 0 }
sub can_show_preferences { return 0 }

sub mark_all_read 	 { return shift->{_message_list}->mark_all_read      }

sub hide { 
    my $s = shift;
    $s->{_message_list}->hide;
    $s->{_message_list}->cleanup if defined($s->{_message_list});
    $s->{_prefs}->set('category_view_'.$s->{_cat}->id(), 'pane_heigth', $s->{_widget}->get_position);
                                                                                                                                                          
    if (! $s->{_prefs}->per_view_pane()) {
          $s->{_prefs}->set('all_views_global', 'pane_heigth',$s->{_widget}->get_position);
    }

}

1;
