package RSSReader::Gtk2UI::Message;

use strict;
use Gtk2::Html2;
use Gnome2::VFS;
use Glib qw(TRUE FALSE);
use Data::Dumper;
use base qw(RSSReader::Logger);

sub new {
   my $c = shift;
   my %a = @_;

   my $s = bless {}, $c;

   $s->{_current_message} = undef;

   $s->{_widget} = Gtk2::VBox->new();
   $s->{title}   = Gtk2::Label->new();
   $s->{title}->set_markup('<b>No message selected</b>');
   $s->{hbox} = new Gtk2::HBox;
   $s->{_widget}->pack_start($s->{hbox},0,0,0);
   $s->{_bbox} = new Gtk2::HButtonBox();
   $s->{_url} = Gtk2::Button->new;
   $s->{_url}->add(Gtk2::Image->new_from_stock('gtk-jump-to','button'));
   $s->{_url}->set_relief('none');
   $s->{_url}->set_sensitive(0);
   $s->{_url}->signal_connect('clicked', sub { RSSReader::Gtk2UI->launch_url($s->{_current_message}->url) });
   $s->{_url}->signal_connect('button-press-event', sub { return unless ($_[1]->button == 3) ;
							  $s->popup_button($s->{_current_message}->url) });
   $s->{_comments} = Gtk2::Button->new;
   $s->{_comments}->add(Gtk2::Image->new_from_stock('syndigator-post-comment','button'));
   $s->{_comments}->set_relief('none');
   $s->{_comments}->signal_connect('clicked', 
   				   sub { 
   					RSSReader::Gtk2UI->launch_url($s->{_current_message}->comments) 
   				   });
   $s->{_comments}->set_sensitive(0);

   $s->{_tooltip} = new Gtk2::Tooltips;
   $s->{_tooltip}->set_tip($s->{_url}, "Open message url in browser.\nRight click to copy url","");
   $s->{_tooltip}->set_tip($s->{_comments}, "Open in browser the url to post a comment to this message.","");

   $s->{hbox}->pack_start($s->{title},1,1,1);
   $s->{hbox}->pack_start($s->{_url},0,0,0);
   $s->{hbox}->pack_start($s->{_comments},0,0,0);

   my $scrolled_win = new Gtk2::ScrolledWindow( undef, undef );
   $scrolled_win->set_policy( 'automatic', 'automatic' );
   $scrolled_win->show();
   $s->{_scrolled_window} = $scrolled_win;

   $s->{_view} = new Gtk2::Html2::View;;
   $s->{_document} = new Gtk2::Html2::Document;
   $s->{_document}->signal_connect (request_url => \&request_url, $s);
   $s->{_document}->signal_connect (link_clicked => sub { RSSReader::Gtk2UI->launch_url($_[1]) } );
   $s->{_view}->set_document($s->{_document});

   $scrolled_win->add($s->{_view});
   $scrolled_win->show_all;
   $s->{_widget}->pack_start($scrolled_win,1,1,1);

   if ($a{message}) {
   	$s->load_message(message=>$a{message});
   }

   $s->{_widget}->show_all;

   return $s;
}


sub popup_button {
    my $s   = shift;
    my $url = shift;

    my @items = (
        [
                '/Open in browser',
                undef,
                sub { RSSReader::Gtk2UI->new->launch_url($url) },
                1,
                '<StockItem>',
                'gtk-jump-to'
        ],
        [
                '/Copy location',
                "",
                sub { RSSReader::Gtk2UI->new->set_clipboard($url); },
                1,
                '<StockItem>',
                'gtk-copy'
        ],
	
    );
    my $factory = Gtk2::ItemFactory->new('Gtk2::Menu', '<messagepopup>', undef);
    $factory->create_items(1, @items);

    my $menu = $factory->get_widget('<messagepopup>');
    $menu->popup(undef, undef, undef,undef, 0, Gtk2->get_current_event_time());

}

sub widget {  return shift->{_widget} }
sub scrolled_window {  return shift->{_scrolled_window} }
sub scroll { 
	my $s = shift;
	print STDERR "Scrolling\n";
	my $adj = $s->{_scrolled_window}->get_vadjustment;
	my $value = $adj->get_value;
	my $top   = $adj->lower;
	my $bottom = $adj->upper;	
	my $p_inc = $adj->page_increment;
	print STDERR " ".$adj->page_size()."  [ $top, $bottom ] -> $value ]\n";
	if ( ($value + $p_inc + $adj->page_size ) >= $bottom ) {
	 print STDERR "  Ultrapassa o final\n";
	 $adj->set_value( $bottom - $adj->page_size );
	} else {
	 $adj->set_value( $value + $p_inc );
	}
	print STDERR " ".$adj->page_size()."  [ $top, $bottom ] -> ".$adj->get_value."]\n";
	return 0 ;
}
sub full_scrolled { 
	my $s = shift;
	my $adj = $s->{_scrolled_window}->get_vadjustment;
        my $value = $adj->get_value;
        my $top   = $adj->lower;
        my $bottom = $adj->upper;
        my $p_inc = $adj->page_increment;
	if ( ($value + $adj->page_size() ) >= $bottom ) {
		return 1;
	}
	return 0;
}	

sub current_message { return shift->{_current_message} }

sub load_message {
	my $s = shift;
	my %a = @_;

	die "Need a message" unless $a{message};

	$s->{_scrolled_window}->get_vadjustment->set_value(0);

	$s->{_current_message} = $a{message};

        $s->{_view}->set_document(undef);
   	$s->{_document} = new Gtk2::Html2::Document;
   	$s->{_document}->signal_connect (request_url => \&request_url, $s);
   	$s->{_document}->signal_connect (link_clicked => sub { RSSReader::Gtk2UI->launch_url($_[1]) } );
	$s->{_view}->set_document($s->{_document});


	my $title_text = '<b>'.$a{message}->title(escaped=>1).'</b> ('.$a{message}->date.')';
	$s->{title}->set_markup($title_text);

	my $text = $a{message}->text;

	if ($a{message}->url) { $s->{_url}->set_sensitive(1) }
	else		      { $s->{_url}->set_sensitive(0) }
	if ($a{message}->comments) { $s->{_comments}->set_sensitive(1) }
	else		           { $s->{_comments}->set_sensitive(0) }

	use Text::Iconv;                                                                                                  

	$s->{_document}->open_stream ("text/html");
	$s->{_document}->write_stream('<html>
	                                  <head>
					     <title>Syndigator Message</title>
				             <meta http-equiv="Content-Type" content="text/html; charset=utf-8">
					  </head><body>'.$text.'</body></html>');
	$s->{_document}->close_stream();
	
	return 1;
}

sub request_url {
        my ($document, $url, $stream, $s) = @_;

	if (not $s->{_gnomevfsinitialized} )  {
                Gnome2::VFS->init;
                $s->{_gnomevfsinitialized} = TRUE;
        }
	
	warn $url;
        if ($url !~ /^http:\/\//) {
                warn "$url is relative";
                my $u = $s->{_current_message}->url;
                warn "message url : $u";
                my ($h) = ( $u =~ /(^(.+?\/\/)?.+?\/)/ );
                                                                                
                $url = $h.'/'.$url;
        }

	warn $url;

	$stream->set_cancel_func(sub { 
				#print STDERR Dumper(\@_);
				$_[0]->{_canceled} = 1;
				});
	# defer to idle for more responsive incremental page loading.
        $s->{_idl} = Glib::Idle->add (sub {
                my $result = vfs_fetch (
                        uri => $url,
                        prepared => sub {
                                my $info = shift;
                                $stream->set_mime_type ($info->get_mime_type);
                        },
                        read_chunk => sub {
				 print STDERR "will write a chunk\n";
				 print STDERR "Found canceled stream: ".$stream->get_pointer."\n";
				 #print STDERR Dumper($stream);
                                 $stream->write ($_[0]) unless $stream->{_canceled};
                                 Gtk2->main_iteration
                                        while Gtk2->events_pending;
                        },
                        #finish => sub { $stream->close; },
                );
                # silently ignore errors.
                warn "# silently ignore errors. -- $result\n";
                return FALSE;
        });
}

sub vfs_fetch {
        my %params = (chunk_size => 1024, @_);
                                                                                                        
        warn "vfs_fetch $params{uri}\n";
                                                                                                        
        my ($result, $handle) = Gnome2::VFS->open ($params{uri}, 'read');
        return $result unless $result eq 'ok';
                                                                                                        
        my (undef, $info) = $handle->get_file_info("default");
                                                                                                        
        $params{prepared}->($info);
                                                                                                        
        do {
                my ($tmp, $nread);
                ($result, $nread, $tmp) = $handle->read ($params{chunk_size});
                $params{read_chunk}->($tmp);
        } while ($result eq 'ok');
                                                                                                        
	#$params{cancel}->();
        #$params{finish}->();

        $result = $handle->close;
}

1;
