package RSSReader::Gtk2UI::Notifiers;
use base qw(RSSReader::Logger);
use RSSReader::Gtk2UI::Preferences;
use Data::Dumper;

#
# Notifiers should notify user of happenings
#

our $_notifiers_registred;

sub register {
	my $c = shift;
	my %a = @_;

	return unless ($a{id} && $a{package});
	$_notifiers_registred->{$a{id}} = $a{package};
	$c->log_debug("Got a notifier register : $a{id} ($a{package})");
	$a{package}->new();
}

sub get_notifier  {
	my $c = shift;
	my $id = shift;

	$c->log_debug("Notifier $id was requested (".$_notifiers_registred->{$id}.")");
	if (exists($_notifiers_registred->{$id}) && defined($_notifiers_registred->{$id}) ) {
		my $_not;
		eval {
		  $_not = ($_notifiers_registred->{$id})->new()
		};
		if ($@) {
		  __PACKAGE__->log_warn("Could not load notifier $id: $@");
		  return undef;
		}
		return $_not;

	}
	$c->log_debug("Notifier does not exist");
	return undef
}

sub get_notifiers {
	my @to_ret;

	foreach ( keys %$_notifiers_registred ) {
		push @to_ret, __PACKAGE__->get_notifier($_);
	}
	return wantarray ? @to_ret : \@to_ret;
}

sub get_option {
  	my $s      = shift;
	my $option = shift;

	my $val = RSSReader::Gtk2UI::Preferences->new->get_notifier_option(id=>$s->id, option=>$option);
	if ((!defined($val)) && ($option eq 'enabled')) { return 0 }

	return $val
}

sub set_option {
	my $s 	   = shift;
	my $option = shift;
	my $val    = shift;

	RSSReader::Gtk2UI::Preferences->new->set_notifier_option(id=>$s->id, option=>$option, value=>$val);
	return 1;
}

sub display_all {
	my $c = shift;
	foreach my $n ( @{$c->get_notifiers} ) {
		print STDERR "Diplaying ".$n->name."\n";
		$n->display;
	}
	return 1;
}

#default_catchall
sub display { }

sub is_enabled { return shift->get_option('enabled') }

sub enable {
        my $s = shift;
        $s->log_debug("I was enabled");

        $s->set_option('enabled', 1) unless $s->is_enabled;
}

sub disable { 
        my $s = shift;
        $s->log_debug("I was disabled");

        $s->set_option('enabled', 0) if $s->is_enabled;
}


#Auto discover
my $myname = __PACKAGE__;
my $me = $myname;
$me =~ s/\:\:/\//g;

foreach my $d (@INC) {
   chomp $d;
   if (-d "$d/$me/") {
     local(*F_DIR);
     opendir(*F_DIR, "$d/$me/");
     while ( my $b = readdir(*F_DIR)) {
         next unless $b =~ /^(.*)\.pm$/;
         __PACKAGE__->log_debug("Found notifier $1"); 
	 my $package_name = __PACKAGE__."::$1";
	 eval "require $package_name";
	 if ($@) { 
		__PACKAGE__->log_error("Error loading notifier $1 : $@");
	 }
     }
   }
}

1;
