package RSSReader::Importer;

use strict;
use Data::Dumper;
use base qw( RSSReader::Logger );

our $_importers_registred;

sub register {
  my $s = shift;
  my %a = @_;

  return unless ($a{id} && $a{package});
  $_importers_registred->{$a{id}} = $a{package};
  $s->log_debug("Got a importer register : $a{id} ($a{package})");
}

sub get_importer  {
	my $c = shift;
	my $id = shift;

	$c->log_debug("Importer $id was requested (".$_importers_registred->{$id}.")");
	if (exists($_importers_registred->{$id}) && defined($_importers_registred->{$id}) ) {
		my $_not;
		eval {
		  $_not = ($_importers_registred->{$id})->new()
		};
		if ($@) {
		  __PACKAGE__->log_warn("Could not load importer $id: $@");
		  return undef;
		}
		return $_not;

	}
	return undef
}

sub get_importers {
	my @to_ret;

	foreach ( keys %$_importers_registred ) {
		push @to_ret, __PACKAGE__->get_importer($_);
	}
	return wantarray ? @to_ret : \@to_ret;
}

#inherit methods
sub add {
  my $s = shift;
  my %a = @_;

  if (!$a{feeds}) {
	$@ = "Need a list of feeds to insert";
	return undef;
  }
  unless (ref($a{feeds}) eq 'ARRAY' ) { $a{feeds} = [ $a{feeds} ] };

  $s->{i_categories} = { };

  #build primary cat struct;
  if ($a{categories}) {
	foreach my $cat (@$a{categories}) {
		$s->{i_categories}->{$cat->{id}} = $cat;
	}
  }

  my @feeds = @{$a{feeds}};
  my $root_cat = RSSReader::Category->new(id=>1);
  $s->{__root_cat}  = RSSReader::Category->create(
                                          label=>$s->name." Imports",
                                          location=>$root_cat
                                         );

  foreach my $feed (@feeds) {
      $s->log_spam("Inserting : $feed->{label} => $feed->{url} ");
      my $cat;
      
      if (!$feed->{parent}) { 
	$cat = $s->{__root_cat};
      } else {
        $s->log_spam("Building cat ".$feed->{parent}."\n");
	$cat = $s->create_cat($s->{i_categories}->{$feed->{parent}});
      }

      RSSReader::Feed->create(
                      label          => $feed->{label},
                      category       => $cat,
                      url            => $feed->{url},
                      fetch_interval => 0
              );
  }
  $s->log_spam("...Done\n");
  return 1;
}

sub create_cat {
  my $s = shift;
  my $cat = shift; 
  $s->log_spam("Should create cat ".Dumper($cat));
  return $s->{i_categories}->{$cat->{id}}->{object} if $s->{i_categories}->{$cat->{id}}->{object};

  my $parent;
  if ($cat->{parent}) {
	$parent = $s->create_cat($s->{i_categories}->{$cat->{parent}});
  }
  else {
        $parent = $s->{__root_cat}; 
  }

  my $cat_obj  = RSSReader::Category->create(
                                          label=>$cat->{label},
                                          location=>$parent
                                         );
  $s->{i_categories}->{$cat->{id}}->{object} = $cat_obj;
  return $cat_obj;
}

#Auto discover
my $myname = __PACKAGE__;
my $me = $myname;
$me =~ s/\:\:/\//g;

foreach my $d (@INC) {
   chomp $d;
   if (-d "$d/$me/") {
     local(*F_DIR);
     opendir(*F_DIR, "$d/$me/");
     while ( my $b = readdir(*F_DIR)) {
         next unless $b =~ /^(.*)\.pm$/;
         __PACKAGE__->log_debug("Found importer $1"); 
	 my $package_name = __PACKAGE__."::$1";
	 eval "require $package_name";
     }
   }
}
	
1;
